@@ -1,3 +1,5 @@
+#' @include tracking-observer.R
+NULL
 
 # Translate metric to value to safe format for REST.
 metric_value_to_rest <- function(value) {
@@ -37,14 +39,17 @@ mlflow_log_metric <- function(key, value, timestamp = NULL, step = NULL, run_id
   timestamp <- cast_nullable_scalar_double(timestamp)
   timestamp <- round(timestamp %||% current_time())
   step <- round(cast_nullable_scalar_double(step) %||% 0)
-  mlflow_rest("runs", "log-metric", client = client, verb = "POST", data = list(
+  data <- list(
     run_uuid = run_id,
     run_id = run_id,
     key = key,
     value = value,
     timestamp = timestamp,
     step = step
-  ))
+  )
+  mlflow_rest("runs", "log-metric", client = client, verb = "POST", data = data)
+  mlflow_register_tracking_event("log_metric", data)
+
   invisible(value)
 }
 
@@ -61,16 +66,16 @@ mlflow_create_run <- function(start_time = NULL, tags = NULL, experiment_id = NU
 
   start_time <- start_time %||% current_time()
 
-  response <- mlflow_rest(
-    "runs", "create",
-    client = client, verb = "POST",
-    data = list(
-      experiment_id = experiment_id,
-      user_id = user_id,
-      start_time = start_time,
-      tags = tags
-    )
+  data <- list(
+    experiment_id = experiment_id,
+    user_id = user_id,
+    start_time = start_time,
+    tags = tags
   )
+  response <- mlflow_rest(
+    "runs", "create", client = client, verb = "POST", data = data
+  )
+  mlflow_register_tracking_event("create_run", data)
 
   mlflow_get_run(run_id = response$run$info$run_uuid, client = client)
 }
@@ -86,9 +91,9 @@ mlflow_delete_run <- function(run_id, client = NULL) {
   if (identical(run_id, mlflow_get_active_run_id()))
     stop("Cannot delete an active run.", call. = FALSE)
   client <- resolve_client(client)
-  mlflow_rest("runs", "delete", client = client, verb = "POST", data = list(
-    run_id = run_id
-  ))
+  data <- list(run_id = run_id)
+  mlflow_rest("runs", "delete", client = client, verb = "POST", data = data)
+  mlflow_register_tracking_event("delete_run", data)
   invisible(NULL)
 }
 
@@ -101,9 +106,10 @@ mlflow_delete_run <- function(run_id, client = NULL) {
 mlflow_restore_run <- function(run_id, client = NULL) {
   run_id <- cast_string(run_id)
   client <- resolve_client(client)
-  mlflow_rest("runs", "restore", client = client, verb = "POST", data = list(
-    run_id = run_id
-  ))
+  data <- list(run_id = run_id)
+  mlflow_rest("runs", "restore", client = client, verb = "POST", data = data)
+  mlflow_register_tracking_event("restore_run", data)
+
   mlflow_get_run(run_id)
 }
 
@@ -149,12 +155,15 @@ mlflow_log_batch <- function(metrics = NULL, params = NULL, tags = NULL, run_id
 
   c(client, run_id) %<-% resolve_client_and_run_id(client, run_id)
 
-  mlflow_rest("runs", "log-batch", client = client, verb = "POST", data = list(
+  data <- list(
     run_id = run_id,
     metrics = metrics,
     params = params,
     tags = tags
-  ))
+  )
+  mlflow_rest("runs", "log-batch", client = client, verb = "POST", data = data)
+  mlflow_register_tracking_event("log_batch", data)
+
   invisible(NULL)
 }
 
@@ -198,12 +207,14 @@ mlflow_set_tag <- function(key, value, run_id = NULL, client = NULL) {
   key <- cast_string(key)
   value <- cast_string(value)
 
-  mlflow_rest("runs", "set-tag", client = client, verb = "POST", data = list(
+  data <- list(
     run_uuid = run_id,
     run_id = run_id,
     key = key,
     value = value
-  ))
+  )
+  mlflow_rest("runs", "set-tag", client = client, verb = "POST", data = data)
+  mlflow_register_tracking_event("set_tag", data)
 
   invisible(NULL)
 }
@@ -222,10 +233,9 @@ mlflow_delete_tag <- function(key, run_id = NULL, client = NULL) {
 
   key <- cast_string(key)
 
-  mlflow_rest("runs", "delete-tag", client = client, verb = "POST", data = list(
-    run_id = run_id,
-    key = key
-  ))
+  data <- list(run_id = run_id, key = key)
+  mlflow_rest("runs", "delete-tag", client = client, verb = "POST", data = data)
+  mlflow_register_tracking_event("delete_tag", data)
 
   invisible(NULL)
 }
@@ -248,12 +258,14 @@ mlflow_log_param <- function(key, value, run_id = NULL, client = NULL) {
   key <- cast_string(key)
   value <- cast_string(value)
 
-  mlflow_rest("runs", "log-parameter", client = client, verb = "POST", data = list(
+  data <- list(
     run_uuid = run_id,
     run_id = run_id,
     key = key,
     value = cast_string(value)
-  ))
+  )
+  mlflow_rest("runs", "log-parameter", client = client, verb = "POST", data = data)
+  mlflow_register_tracking_event("log_param", data)
 
   invisible(value)
 }
@@ -368,13 +380,15 @@ mlflow_list_artifacts <- function(path = NULL, run_id = NULL, client = NULL) {
 }
 
 mlflow_set_terminated <- function(status, end_time, run_id, client) {
-
-  response <- mlflow_rest("runs", "update", verb = "POST", client = client, data = list(
+  data <- list(
     run_uuid = run_id,
     run_id = run_id,
     status = status,
     end_time = end_time
-  ))
+  )
+  response <- mlflow_rest("runs", "update", verb = "POST", client = client, data = data)
+  mlflow_register_tracking_event("set_terminated", data)
+
   mlflow_get_run(client = client, run_id = response$run_info$run_uuid)
 }
 
@@ -566,6 +580,7 @@ mlflow_start_run <- function(run_id = NULL, experiment_id = NULL, start_time = N
     do.call(mlflow_create_run, args)
   }
   mlflow_set_active_run_id(mlflow_id(run))
+  mlflow_set_experiment(experiment_id = args$experiment_id)
   run
 }
 
