@@ -7,9 +7,9 @@ Python (native) `pickle <https://scikit-learn.org/stable/modules/model_persisten
 
 :py:mod:`mlflow.pyfunc`
     Produced for use by generic pyfunc-based deployment tools and batch inference.
+    NOTE: The `mlflow.pyfunc` flavor is only added for scikit-learn models that define `predict()`,
+    since `predict()` is required for pyfunc model inference.
 """
-import functools
-import gorilla
 import os
 import logging
 import pickle
@@ -27,10 +27,17 @@ from mlflow.models.utils import ModelInputExample, _save_example
 from mlflow.protos.databricks_pb2 import INVALID_PARAMETER_VALUE, INTERNAL_ERROR
 from mlflow.protos.databricks_pb2 import RESOURCE_ALREADY_EXISTS
 from mlflow.tracking.artifact_utils import _download_artifact_from_uri
+from mlflow.utils import gorilla
 from mlflow.utils.annotations import experimental
 from mlflow.utils.environment import _mlflow_conda_env
 from mlflow.utils.model_utils import _get_flavor_configuration
-from mlflow.utils.autologging_utils import try_mlflow_log
+from mlflow.utils.autologging_utils import (
+    try_mlflow_log,
+    wrap_patch,
+    INPUT_EXAMPLE_SAMPLE_ROWS,
+    resolve_input_example_and_signature,
+)
+from mlflow.tracking._model_registry import DEFAULT_AWAIT_MAX_SLEEP_SECONDS
 
 FLAVOR_NAME = "sklearn"
 
@@ -71,7 +78,12 @@ def save_model(
     input_example: ModelInputExample = None,
 ):
     """
-    Save a scikit-learn model to a path on the local file system.
+    Save a scikit-learn model to a path on the local file system. Produces an MLflow Model
+    containing the following flavors:
+
+        - :py:mod:`mlflow.sklearn`
+        - :py:mod:`mlflow.pyfunc`. NOTE: This flavor is only included for scikit-learn models
+          that define `predict()`, since `predict()` is required for pyfunc model inference.
 
     :param sk_model: scikit-learn model to be saved.
     :param path: Local path where the model is to be saved.
@@ -213,9 +225,15 @@ def log_model(
     registered_model_name=None,
     signature: ModelSignature = None,
     input_example: ModelInputExample = None,
+    await_registration_for=DEFAULT_AWAIT_MAX_SLEEP_SECONDS,
 ):
     """
-    Log a scikit-learn model as an MLflow artifact for the current run.
+    Log a scikit-learn model as an MLflow artifact for the current run. Produces an MLflow Model
+    containing the following flavors:
+
+        - :py:mod:`mlflow.sklearn`
+        - :py:mod:`mlflow.pyfunc`. NOTE: This flavor is only included for scikit-learn models
+          that define `predict()`, since `predict()` is required for pyfunc model inference.
 
     :param sk_model: scikit-learn model to be saved.
     :param artifact_path: Run-relative artifact path.
@@ -264,7 +282,9 @@ def log_model(
                           model. The given example will be converted to a Pandas DataFrame and then
                           serialized to json using the Pandas split-oriented format. Bytes are
                           base64-encoded.
-
+    :param await_registration_for: Number of seconds to wait for the model version to finish
+                            being created and is in ``READY`` status. By default, the function
+                            waits for five minutes. Specify 0 or None to skip waiting.
 
 
     .. code-block:: python
@@ -296,6 +316,7 @@ def log_model(
         registered_model_name=registered_model_name,
         signature=signature,
         input_example=input_example,
+        await_registration_for=await_registration_for,
     )
 
 
@@ -511,7 +532,7 @@ class _SklearnTrainingSession(object):
 
 
 @experimental
-def autolog():
+def autolog(log_input_examples=False, log_model_signatures=True):
     """
     Enables autologging for scikit-learn estimators.
 
@@ -531,6 +552,58 @@ def autolog():
       **Metrics**
         - A training score obtained by ``estimator.score``. Note that the training score is
           computed using parameters given to ``fit()``.
+        - Common metrics for classifier:
+
+          - `precision score`_
+
+          .. _precision score:
+              https://scikit-learn.org/stable/modules/generated/sklearn.metrics.precision_score.html
+
+          - `recall score`_
+
+          .. _recall score:
+              https://scikit-learn.org/stable/modules/generated/sklearn.metrics.recall_score.html
+
+          - `f1 score`_
+
+          .. _f1 score:
+              https://scikit-learn.org/stable/modules/generated/sklearn.metrics.f1_score.html
+
+          - `accuracy score`_
+
+          .. _accuracy score:
+              https://scikit-learn.org/stable/modules/generated/sklearn.metrics.accuracy_score.html
+
+          If the classifier has method ``predict_proba``, we additionally log:
+
+          - `log loss`_
+
+          .. _log loss:
+              https://scikit-learn.org/stable/modules/generated/sklearn.metrics.log_loss.html
+
+          - `roc auc score`_
+
+          .. _roc auc score:
+              https://scikit-learn.org/stable/modules/generated/sklearn.metrics.roc_auc_score.html
+
+        - Common metrics for regressor:
+
+          - `mean squared error`_
+
+          .. _mean squared error:
+              https://scikit-learn.org/stable/modules/generated/sklearn.metrics.mean_squared_error.html
+
+          - root mean squared error
+
+          - `mean absolute error`_
+
+          .. _mean absolute error:
+              https://scikit-learn.org/stable/modules/generated/sklearn.metrics.mean_absolute_error.html
+
+          - `r2 score`_
+
+          .. _r2 score:
+              https://scikit-learn.org/stable/modules/generated/sklearn.metrics.r2_score.html
 
       **Tags**
         - An estimator class name (e.g. "LinearRegression").
@@ -538,7 +611,9 @@ def autolog():
           (e.g. "sklearn.linear_model._base.LinearRegression").
 
       **Artifacts**
-        - A fitted estimator (logged by :py:func:`mlflow.sklearn.log_model()`).
+        - An MLflow Model with the :py:mod:`mlflow.sklearn` flavor containing a fitted estimator
+          (logged by :py:func:`mlflow.sklearn.log_model()`). The Model also contains the
+          :py:mod:`mlflow.pyfunc` flavor when the scikit-learn estimator defines `predict()`.
 
     **How does autologging work for meta estimators?**
       When a meta estimator (e.g. `Pipeline`_, `GridSearchCV`_) calls ``fit()``, it internally calls
@@ -608,7 +683,11 @@ def autolog():
         #  'normalize': 'False'}
 
         pprint(metrics)
-        # {'training_score': 1.0}
+        # {'training_score': 1.0,
+           'training_mae': 2.220446049250313e-16,
+           'training_mse': 1.9721522630525295e-31,
+           'training_r2_score': 1.0,
+           'training_rmse': 4.440892098500626e-16}
 
         pprint(tags)
         # {'estimator_class': 'sklearn.linear_model._base.LinearRegression',
@@ -616,6 +695,15 @@ def autolog():
 
         pprint(artifacts)
         # ['model/MLmodel', 'model/conda.yaml', 'model/model.pkl']
+
+    :param log_input_examples: If ``True``, input examples from training datasets are collected and
+                               logged along with scikit-learn model artifacts during training. If
+                               ``False``, input examples are not logged.
+    :param log_model_signatures: If ``True``,
+                                 :py:class:`ModelSignatures <mlflow.models.ModelSignature>`
+                                 describing model inputs and outputs are collected and logged along
+                                 with scikit-learn model artifacts during training. If ``False``,
+                                 signatures are not logged.
     """
     import pandas as pd
     import sklearn
@@ -626,6 +714,7 @@ def autolog():
         _is_supported_version,
         _chunk_dict,
         _get_args_for_score,
+        _log_specialized_estimator_content,
         _get_Xy,
         _all_estimators,
         _truncate_dict,
@@ -652,16 +741,21 @@ def autolog():
             stacklevel=2,
         )
 
-    def fit_mlflow(self, func_name, *args, **kwargs):
+    def fit_mlflow(self, clazz, func_name, *args, **kwargs):
+        """
+        Autologging function that performs model training by executing the training method
+        referred to be `func_name` on the instance of `clazz` referred to by `self` & records
+        MLflow parameters, metrics, tags, and artifacts to a corresponding MLflow Run.
+        """
         should_start_run = mlflow.active_run() is None
         if should_start_run:
             try_mlflow_log(mlflow.start_run)
 
         _log_pretraining_metadata(self, *args, **kwargs)
 
-        original_fit = gorilla.get_original_attribute(self, func_name)
+        original_fit = gorilla.get_original_attribute(clazz, func_name)
         try:
-            fit_output = original_fit(*args, **kwargs)
+            fit_output = original_fit(self, *args, **kwargs)
         except Exception as e:
             if should_start_run:
                 try_mlflow_log(mlflow.end_run, RunStatus.to_string(RunStatus.FAILED))
@@ -730,23 +824,35 @@ def autolog():
             else:
                 try_mlflow_log(mlflow.log_metric, "training_score", training_score)
 
-        input_example = None
-        signature = None
-        if hasattr(estimator, "predict"):
-            try:
-                # Fetch an input example using the first several rows of the array-like
-                # training data supplied to the training routine (e.g., `fit()`)
-                SAMPLE_ROWS = 5
-                fit_arg_names = _get_arg_names(estimator.fit)
-                X_var_name, y_var_name = fit_arg_names[:2]
-                input_example = _get_Xy(args, kwargs, X_var_name, y_var_name)[0][:SAMPLE_ROWS]
+        # log common metrics and artifacts for estimators (classifier, regressor)
+        _log_specialized_estimator_content(estimator, mlflow.active_run().info.run_id, args, kwargs)
 
-                model_output = estimator.predict(input_example)
-                signature = infer_signature(input_example, model_output)
-            except Exception as e:  # pylint: disable=broad-except
-                input_example = None
-                msg = "Failed to infer an input example and model signature: " + str(e)
-                _logger.warning(msg)
+        def get_input_example():
+            # Fetch an input example using the first several rows of the array-like
+            # training data supplied to the training routine (e.g., `fit()`)
+            fit_arg_names = _get_arg_names(estimator.fit)
+            X_var_name, y_var_name = fit_arg_names[:2]
+            input_example = _get_Xy(args, kwargs, X_var_name, y_var_name)[0][
+                :INPUT_EXAMPLE_SAMPLE_ROWS
+            ]
+            return input_example
+
+        def infer_model_signature(input_example):
+            if not hasattr(estimator, "predict"):
+                raise Exception(
+                    "the trained model does not specify a `predict` function, "
+                    + "which is required in order to infer the signature"
+                )
+
+            return infer_signature(input_example, estimator.predict(input_example))
+
+        input_example, signature = resolve_input_example_and_signature(
+            get_input_example,
+            infer_model_signature,
+            log_input_examples,
+            log_model_signatures,
+            _logger,
+        )
 
         try_mlflow_log(
             log_model,
@@ -766,6 +872,9 @@ def autolog():
                     input_example=input_example,
                 )
 
+            if hasattr(estimator, "best_score_"):
+                try_mlflow_log(mlflow.log_metric, "best_cv_score", estimator.best_score_)
+
             if hasattr(estimator, "best_params_"):
                 best_params = {
                     "best_{param_name}".format(param_name=param_name): param_value
@@ -804,31 +913,59 @@ def autolog():
                     )
                     _logger.warning(msg)
 
-    def patched_fit(self, func_name, *args, **kwargs):
+    def patched_fit(self, clazz, func_name, *args, **kwargs):
         """
-        To be applied to a sklearn model class that defines a `fit` method and
-        inherits from `BaseEstimator` (thereby defining the `get_params()` method)
-        """
-        with _SklearnTrainingSession(clazz=self.__class__, allow_children=False) as t:
-            if t.should_log():
-                return fit_mlflow(self, func_name, *args, **kwargs)
-            else:
-                original_fit = gorilla.get_original_attribute(self, func_name)
-                return original_fit(*args, **kwargs)
+        Autologging patch function to be applied to a sklearn model class that defines a `fit`
+        method and inherits from `BaseEstimator` (thereby defining the `get_params()` method)
 
-    def create_patch_func(func_name):
+        :param clazz: The scikit-learn model class to which this patch function is being applied for
+                      autologging (e.g., `sklearn.linear_model.LogisticRegression`)
+        :param func_name: The function name on the specified `clazz` that this patch is overriding
+                          for autologging (e.g., specify "fit" in order to indicate that
+                          `sklearn.linear_model.LogisticRegression.fit()` is being patched)
+        """
+        with _SklearnTrainingSession(clazz=clazz, allow_children=False) as t:
+            if t.should_log():
+                return fit_mlflow(self, clazz, func_name, *args, **kwargs)
+            else:
+                original_fit = gorilla.get_original_attribute(clazz, func_name)
+                return original_fit(self, *args, **kwargs)
+
+    def create_patch_func(clazz, func_name):
         def f(self, *args, **kwargs):
-            return patched_fit(self, func_name, *args, **kwargs)
+            return patched_fit(self, clazz, func_name, *args, **kwargs)
 
         return f
 
-    patch_settings = gorilla.Settings(allow_hit=True, store_hit=True)
     _, estimators_to_patch = zip(*_all_estimators())
     # Ensure that relevant meta estimators (e.g. GridSearchCV, Pipeline) are selected
     # for patching if they are not already included in the output of `all_estimators()`
     estimators_to_patch = set(estimators_to_patch).union(
         set(_get_meta_estimators_for_autologging())
     )
+    # Exclude certain preprocessing & feature manipulation estimators from patching. These
+    # estimators represent data manipulation routines (e.g., normalization, label encoding)
+    # rather than ML algorithms. Accordingly, we should not create MLflow runs and log
+    # parameters / metrics for these routines, unless they are captured as part of an ML pipeline
+    # (via `sklearn.pipeline.Pipeline`)
+    excluded_module_names = [
+        "sklearn.preprocessing",
+        "sklearn.impute",
+        "sklearn.feature_extraction",
+        "sklearn.feature_selection",
+    ]
+
+    estimators_to_patch = [
+        estimator
+        for estimator in estimators_to_patch
+        if not any(
+            [
+                estimator.__module__.startswith(excluded_module_name)
+                for excluded_module_name in excluded_module_names
+            ]
+        )
+    ]
+
     for class_def in estimators_to_patch:
         for func_name in ["fit", "fit_transform", "fit_predict"]:
             if hasattr(class_def, func_name):
@@ -851,10 +988,5 @@ def autolog():
                 if isinstance(original, property):
                     continue
 
-                patch_func = create_patch_func(func_name)
-                # TODO(harupy): Package this wrap & patch routine into a utility function so we can
-                # reuse it in other autologging integrations.
-                # preserve original function attributes
-                patch_func = functools.wraps(original)(patch_func)
-                patch = gorilla.Patch(class_def, func_name, patch_func, settings=patch_settings)
-                gorilla.apply(patch)
+                patch_func = create_patch_func(class_def, func_name)
+                wrap_patch(class_def, func_name, patch_func)
